{% load i18n %}
<h3>{% blocktrans with title as filter_title %} By {{ filter_title }} {% endblocktrans %}</h3>
<select onchange="window.location=window.location.pathname+this.options[this.selectedIndex].value">
{% for choice in choices %}
    <option{% if choice.selected %} selected="selected"{% endif %} value="{{ choice.query_string|iriencode }}">{{ choice.display }}</option>
{% endfor %}
</select>
