{% load i18n i18n_tags %}
<li>
    <div id="languages">
        {% for language in languages %}
        <span class="language">
            <a href="{% page_language_url language.0 %}">
                <img src="{{ MEDIA_URL }}fluo/flags/{{ language.0 }}.gif" name="language_{{ language.0 }}" alt="{{ language.1 }}"/>
            </a>
        </span>
        {% endfor %}
    </div>
</li>

