{% load i18n i18n_tags %}
<div id="flags" style="float: right;">
    {% for lang in languages %}
    <span style="float: right; padding-right: 4px; vertical-align: middle;">
        <a href="{% page_language_url language.0 %}" method="post">
            <img src="{{ MEDIA_URL }}fluo/flags/{{ language.0 }}.gif" name="language_{{ language.0 }}" alt="{{ language.1 }}"/>
        </a>
    </span>
    {% endfor %}
</div>
